/*
 *  IBM Confidential OCO Source Materials
 *
 *  5725-G92 Copyright IBM Corp. 2011, 2012
 *
 *  The source code for this program is not published or otherwise
 *  divested of its trade secrets, irrespective of what has
 *  been deposited with the U.S. Copyright Office.
 *
 */

/*
 * Program that displays the numeric id of a group (name or number).
 * Examples:
 *   $ ./gid haible
 *   1000
 *   $ ./gid 1000
 *   1000
 *
 * @author Bruno Haible
 */

#include <sys/types.h>
#include <grp.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int
main (int argc, char *argv[])
{
  const char *group_name;
  int numeric;

  if (argc != 2) {
    fprintf (stderr, "Usage: gid group\n");
    exit (1);
  }
  group_name = argv[1];

  if (strcmp (group_name, "") == 0) {
    fprintf (stderr, "gid: invalid empty group\n");
    exit (1);
  }

  {
    const char *p;
    numeric = 1;
    for (p = group_name; *p != '\0'; p++)
      if (!(*p >= '0' && *p <= '9'))
        numeric = 0;
  }
  if (numeric) {
    printf ("%s\n", group_name);
  } else {
    struct group *gr = getgrnam (group_name);
    if (gr == NULL) {
      fprintf (stderr, "gid: unknown group '%s'\n", group_name);
      exit (1);
    }
    printf ("%u\n", gr->gr_gid);
  }

  exit (0);
}

/*
 * For Emacs M-x compile.
 * Local Variables:
 * compile-command: "gcc -m32 -Wall -O -o ../../../../bin.linux-x86/gid.linux-x86.bin gid.c"
 * End:
 */
