/*
* Licensed Materials - Property of IBM
* 5725-G92 (C) Copyright IBM Corp. 2006, 2012. All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

/**
 * welcomeCommon.js
 * The common functions for the specific welcome page implementations.
 * Specific implementations normally override the accessGranted function. 
 */    

WL.AbstractWelcomeMgr = Class.create({    
    // Check access response after this amount of milliseconds.
    // Notice: The welcome page display is delayed for this amount of 
    // milliseconds, but if timer is too short - the success reply might 
    // not arrive on time.
    STARTUP_ACCESS_CHECK_TIMER : 3000,  
    
    // The access check interval - if a sibling gadget logs in, the 
    // gadget will identify it has access in this max milliseconds.
    ACCESS_CHECK_INTERVAL_MILIS : 5000,
    
    /** NOTICE: Server rely on these parameter names */
    AUTH_HANDLER_PATH              : 'authentication',    
    PARAM_NAME_PARENT_URL          : 'parent',
    PARAM_NAME_ACTION              : 'action',
    PARAM_VALUE_POPUP_ACTION       : 'popup',
    PARAM_VALUE_FULLSCREEN_ACTION  :'fullscreen', 
    PARAM_VALUE_CHECK_ACCESS       : 'test',
    PARAM_VALUE_CHECK_ACCESS_IMAGE : 'test_img',
    CHECK_ACCESS_SUCCESS_RESPONSE  :'success', 
    
    messages : {
        enter_en : 'Enter',
        enter_en_us : 'Enter',
        enter_es : 'Entrar',
        // Unicode for Hamshech ...
        enter_he : '\u05D4\u05DE\u05E9\u05DA'
        // Add here more languages ...        
    },    
    
    options : null,
    busyIndicator : null,
    
    isPostMessageSupported : function(){
        return (typeof window.postMessage !== 'undefined' || typeof document.postMessage !== 'undefined');
    },
    
    initialize : function (mainFileName, options) {      
        this.mainFile = mainFileName;
        this.options  = options;
        this.oTimeout = null;
        
        this.busyIndicator = new WL.BusyIndicator('content');
        
        // Handle resize event
        Event.observe(document.onresize ? document : window, "resize", this.adjustWindow.bind(this));
        
        //If no message arrives after X seconds, deny access..
        this.oTimeout = window.setTimeout(this.accessDenied.bind(this), this.STARTUP_ACCESS_CHECK_TIMER);
                
        $('enterButton').href = this.getGadgetURL();
        window.setTimeout(this.initAccessCheck.bind(this), 1);
        this.busyIndicator.show();
        
        this.handleI18nMessages();
    },
    
    loadTestImage : function () {
        var img = document.createElement('img');
    	img.onload = this.accessGranted.bind(this);
        img.src = this.getAuthenticationPathImage();
    },
    
    accessGranted : function () {
        alert("Function must be implemented by the specific welcome implementation");
    },
    
    accessDenied : function () {
    	this.displayWelcomePage();
        this.oTimeout = window.setTimeout(this.accessDenied.bind(this), this.ACCESS_CHECK_INTERVAL_MILIS);
        this.loadTestImage();
    },
    
    displayWelcomePage : function (){
        this.busyIndicator.hide();
        $("mainTable").style.visibility = 'visible';    
    },
    
    getGadgetURL : function() {
        var params = ('' + window.location).toQueryParams();
        //var containerParentURL = queryString(this.PARAM_NAME_PARENT_URL, window.location);
        var gadgetURL = this.mainFile;
        if (gadgetURL.indexOf("?") == -1) {
            gadgetURL += "?";
        } else {
            gadgetURL += "&";
        }
        
        gadgetURL += this.PARAM_NAME_PARENT_URL + '=' + encodeURIComponent(params[this.PARAM_NAME_PARENT_URL]);

        return gadgetURL;
    },
    
    clearTimer : function () {
        if (this.oTimeout !== null) {
            window.clearTimeout(this.oTimeout);
            this.oTimeout = null;
        }
    },
    
    initAccessCheck : function () {
        this.clearTimer();
        this.oTimeout = window.setTimeout(this.accessDenied.bind(this), this.ACCESS_CHECK_INTERVAL_MILIS);
        this.loadTestImage();
    },    
            
    getAuthenticationPathImage : function (){
        var path = '';
        var lastDirIndex = this.mainFile.lastIndexOf("/");
        if (lastDirIndex > 0) {
            path += this.mainFile.substring(0, lastDirIndex);
        }
        path += '/' + this.AUTH_HANDLER_PATH + '/?' +  
            this.PARAM_NAME_ACTION + '=' + this.PARAM_VALUE_CHECK_ACCESS_IMAGE + "&r=" + Math.random();
            
        return path;    
    },
    
    getAuthenticationActionPath : function (authActionType){
        var path = '';
        var lastDirIndex = this.mainFile.lastIndexOf("/");
        if(lastDirIndex > 0) {
            path += this.mainFile.substring(0, lastDirIndex);
        }
        path += '/' + this.AUTH_HANDLER_PATH + '/?' +  
            this.PARAM_NAME_ACTION + '=' + authActionType + '&' + 
            this.PARAM_NAME_PARENT_URL + '=' + this.getParentURL();
            
        return path;    
    },
    
    getParentURL : function(){        
        return encodeURIComponent(this.options.dummyHTMLURL);                  
    },
    
    handleI18nMessages : function () {        
        var lang = 'en';
        if (! Object.isUndefined(navigator.userLanguage)) {
            lang = navigator.userLanguage;
        } else if (! Object.isUndefined(navigator.language)) {
            lang = navigator.language;
        }
        lang = lang.toLowerCase().sub('-', '_');
         
        var enterLabel = this.messages.enter_en;
        if (! Object.isUndefined(this.messages['enter_' + lang])){
            enterLabel = this.messages['enter_' + lang];
        }
        $('enterLabel').update(enterLabel);                
    },
    
    //...... Handling Minimized/Docking Gadget Display ........
    
    // The minimum window width for displaying the welcome page
    // instead of a minimized 'docked' mode.
    minWindowWidth : 200,
    
    filterResults : function (win, docel, body) {
        var result = win ? win : 0;
        if (docel && (!result || (result > docel))){
            result = docel;
        }
        return body && (!result || (result > body)) ? body : result;
    },
    
    clientWidth : function () {
        return this.filterResults(
                window.innerWidth ? window.innerWidth : 0,
                document.documentElement ? document.documentElement.clientWidth : 0,
                document.body ? document.body.clientWidth : 0);
    },
    
    adjustWindow : function () {
        if (this.clientWidth() >= this.minWindowWidth) {
            $('expandWindow').className = "hide";
            $('content').className = "show";
        }
        else {
            $('content').className = "hide";
            $('expandWindow').className = "show";
        }
    }    
});

WL.EmbeddedWelcomeMgr = Class.create(WL.AbstractWelcomeMgr, {
    initialize : function ($super, mainFile, options){
        $super(mainFile, options);                
    },
    
    accessGranted : function () {
        this.clearTimer();
        window.location = this.getGadgetURL(); 
    }
});

WL.FullscreenWelcomeMgr = Class.create(WL.AbstractWelcomeMgr, {

    initialize : function ($super, mainFile, options){
        $super(mainFile, options);
        $('enterButton').target = '_top';
        $('enterButton').href = this.getAuthenticationActionPath(this.PARAM_VALUE_FULLSCREEN_ACTION);        
    },

    accessGranted : function() {
        this.clearTimer();
        window.location = this.getGadgetURL();
    }    
});

WL.PopupWelcomeMgr = Class.create(WL.AbstractWelcomeMgr, {
    
    POPUP_STATE_CHECK_TIME : 1000,    
    loginPopupWindow : null,
    isPopupOpened : false,
    
    initialize : function ($super, mainFile, options){
        $super(mainFile, options);
        $('enterButton').observe('click', this.openPopup.bindAsEventListener(this));
    },
    
    openPopup : function (event) {
        var loginURL = this.getAuthenticationActionPath(this.PARAM_VALUE_POPUP_ACTION);            
        this.loginPopupWindow = window.open(
            loginURL, 
            "loginPopupWindow", 
            "height=" + this.options.popupHeight + ",width=" + this.options.popupWidth + "," +
            "location=1,status=0,toolbar=0,resizable=0, scrollbars=0,menubar=0");
		
        // IE8 - Vista: protected mode - no handle to window, need to check access to server
        this.isPopupOpened = true;
        if (this.loginPopupWindow === null) {
            this.clearTimer();
            this.loadTestImage();
            this.oTimeout = window.setTimeout(this.checkAccess.bind(this), this.STARTUP_ACCESS_CHECK_TIMER);
        } 
        else {
            try {
                this.loginPopupWindow.opener = self;
            } 
            catch(e) { 
                // Carry on,  no harm done (the opener is used in the reply to set focus on it).
            }
            // Prevent default action - href ...
        }
        Event.stop(event);
    },
    
    accessDenied : function ($super) {
        if (this.loginPopupWindow === null && this.isPopupOpened) {
            this.clearTimer();
            this.loadTestImage();
            this.oTimeout = window.setTimeout(this.checkAccess.bind(this), this.STARTUP_ACCESS_CHECK_TIMER);
        } 
        else {            
            $super();
        }
    },
    
    accessGranted : function () {
        if (this.loginPopupWindow !== null && !this.loginPopupWindow.closed) {
           this.loginPopupWindow.close();
        }
        this.clearTimer();        
        window.location = this.getGadgetURL();
        try {
            window.focus();
        }
        catch(e){}
    }
});

var welcomeMgr = null;
function init(loginDisplayType, mainFilePath, options){     
    switch (loginDisplayType){
    case 'embedded':
        welcomeMgr = new WL.EmbeddedWelcomeMgr(mainFilePath, options);
        break;
        
    case 'popup':
        welcomeMgr = new WL.PopupWelcomeMgr(mainFilePath, options);
        break;
        
    case 'fullscreen':
        welcomeMgr = new WL.FullscreenWelcomeMgr(mainFilePath, options);
        break;            
    }
}
